# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import Any, Dict, Optional, Union

from stytch.consumer.models.fraud_verdict_reasons import (
    ListResponse,
    OverrideRequestAction,
    OverrideResponse,
)
from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient


class VerdictReasons:
    def __init__(
        self, api_base: ApiBase, sync_client: SyncClient, async_client: AsyncClient
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client

    def override(
        self,
        verdict_reason: str,
        override_action: Union[OverrideRequestAction, str],
        override_description: Optional[str] = None,
    ) -> OverrideResponse:
        """Use this endpoint to override the action returned for a specific verdict reason during a fingerprint lookup. For example, Stytch Device Fingerprinting returns a `CHALLENGE` verdict action by default for the verdict reason `VIRTUAL_MACHINE`. You can use this endpoint to override that reason to return an `ALLOW` verdict instead if you expect many legitimate users to be using a browser that runs in a virtual machine.

        Fields:
          - verdict_reason: The verdict reason that you wish to override. For a list of possible reasons to override, see [Warning Flags (Verdict Reasons)](https://stytch.com/docs/docs/fraud/guides/device-fingerprinting/reference/warning-flags-verdict-reasons). You may not override the `RULE_MATCH` reason.
          - override_action: The action that you want to be returned for the specified verdict reason. The override action must be one of `ALLOW`, `BLOCK`, or `CHALLENGE`.
          - override_description: An optional description for the verdict reason override.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "verdict_reason": verdict_reason,
            "override_action": override_action,
        }
        if override_description is not None:
            data["override_description"] = override_description

        url = self.api_base.url_for("/v1/verdict_reasons/override", data)
        res = self.sync_client.post(url, data, headers)
        return OverrideResponse.from_json(res.response.status_code, res.json)

    async def override_async(
        self,
        verdict_reason: str,
        override_action: OverrideRequestAction,
        override_description: Optional[str] = None,
    ) -> OverrideResponse:
        """Use this endpoint to override the action returned for a specific verdict reason during a fingerprint lookup. For example, Stytch Device Fingerprinting returns a `CHALLENGE` verdict action by default for the verdict reason `VIRTUAL_MACHINE`. You can use this endpoint to override that reason to return an `ALLOW` verdict instead if you expect many legitimate users to be using a browser that runs in a virtual machine.

        Fields:
          - verdict_reason: The verdict reason that you wish to override. For a list of possible reasons to override, see [Warning Flags (Verdict Reasons)](https://stytch.com/docs/docs/fraud/guides/device-fingerprinting/reference/warning-flags-verdict-reasons). You may not override the `RULE_MATCH` reason.
          - override_action: The action that you want to be returned for the specified verdict reason. The override action must be one of `ALLOW`, `BLOCK`, or `CHALLENGE`.
          - override_description: An optional description for the verdict reason override.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "verdict_reason": verdict_reason,
            "override_action": override_action,
        }
        if override_description is not None:
            data["override_description"] = override_description

        url = self.api_base.url_for("/v1/verdict_reasons/override", data)
        res = await self.async_client.post(url, data, headers)
        return OverrideResponse.from_json(res.response.status, res.json)

    def list(
        self,
        overrides_only: Optional[bool] = None,
    ) -> ListResponse:
        """Get the list of verdict reasons returned by the Stytch Device Fingerprinting product along with their default actions and any overrides you may have defined. This is not an exhaustive list of verdict reasons, but it contains all verdict reasons that you may set an override on.

        For a full list of possible verdict reasons, see [Warning Flags (Verdict Reasons)](https://stytch.com/docs/docs/fraud/guides/device-fingerprinting/reference/warning-flags-verdict-reasons).

        Fields:
          - overrides_only: Whether to return only verdict reasons that have overrides set. Defaults to false.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {}
        if overrides_only is not None:
            data["overrides_only"] = overrides_only

        url = self.api_base.url_for("/v1/verdict_reasons/list", data)
        res = self.sync_client.post(url, data, headers)
        return ListResponse.from_json(res.response.status_code, res.json)

    async def list_async(
        self,
        overrides_only: Optional[bool] = None,
    ) -> ListResponse:
        """Get the list of verdict reasons returned by the Stytch Device Fingerprinting product along with their default actions and any overrides you may have defined. This is not an exhaustive list of verdict reasons, but it contains all verdict reasons that you may set an override on.

        For a full list of possible verdict reasons, see [Warning Flags (Verdict Reasons)](https://stytch.com/docs/docs/fraud/guides/device-fingerprinting/reference/warning-flags-verdict-reasons).

        Fields:
          - overrides_only: Whether to return only verdict reasons that have overrides set. Defaults to false.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {}
        if overrides_only is not None:
            data["overrides_only"] = overrides_only

        url = self.api_base.url_for("/v1/verdict_reasons/list", data)
        res = await self.async_client.post(url, data, headers)
        return ListResponse.from_json(res.response.status, res.json)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!


from typing import Optional

import aiohttp
import jwt

from stytch.consumer.api.connected_apps import ConnectedApp
from stytch.consumer.api.crypto_wallets import CryptoWallets
from stytch.consumer.api.fraud import Fraud
from stytch.consumer.api.idp import IDP
from stytch.consumer.api.impersonation import Impersonation
from stytch.consumer.api.m2m import M2M
from stytch.consumer.api.magic_links import MagicLinks
from stytch.consumer.api.oauth import OAuth
from stytch.consumer.api.otp import OTPs
from stytch.consumer.api.passwords import Passwords
from stytch.consumer.api.policy_cache import PolicyCache
from stytch.consumer.api.project import Project
from stytch.consumer.api.rbac import RBAC
from stytch.consumer.api.sessions import Sessions
from stytch.consumer.api.totps import TOTPs
from stytch.consumer.api.users import Users
from stytch.consumer.api.webauthn import WebAuthn
from stytch.core.client_base import ClientBase


class Client(ClientBase):
    """
    Stytch API Python client.

    Learn more at https://stytch.com/docs
    """

    def __init__(
        self,
        project_id: str,
        secret: str,
        environment: Optional[str] = None,
        suppress_warnings: bool = False,
        async_session: Optional[aiohttp.ClientSession] = None,
        fraud_environment: Optional[str] = None,
        custom_base_url: Optional[str] = None,
    ):
        super().__init__(
            project_id=project_id,
            secret=secret,
            environment=environment,
            suppress_warnings=suppress_warnings,
            async_session=async_session,
            fraud_environment=fraud_environment,
            custom_base_url=custom_base_url,
        )

        policy_cache = PolicyCache(
            RBAC(
                api_base=self.api_base,
                sync_client=self.sync_client,
                async_client=self.async_client,
            )
        )

        self.connected_app = ConnectedApp(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )
        self.crypto_wallets = CryptoWallets(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )
        self.fraud = Fraud(
            api_base=self.fraud_api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )
        self.impersonation = Impersonation(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )
        self.m2m = M2M(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
            jwks_client=self.jwks_client,
            project_id=project_id,
        )
        self.magic_links = MagicLinks(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )
        self.oauth = OAuth(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )
        self.otps = OTPs(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )
        self.passwords = Passwords(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )
        self.project = Project(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )
        self.rbac = RBAC(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )
        self.sessions = Sessions(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
            jwks_client=self.jwks_client,
            project_id=project_id,
            policy_cache=policy_cache,
        )
        self.totps = TOTPs(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )
        self.users = Users(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )
        self.webauthn = WebAuthn(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )
        self.idp = IDP(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
            jwks_client=self.jwks_client,
            project_id=project_id,
            policy_cache=policy_cache,
        )

    def get_jwks_client(self, project_id: str) -> jwt.PyJWKClient:
        data = {"project_id": project_id}
        url = self.api_base.url_for("/v1/sessions/jwks/{project_id}", data)
        return jwt.PyJWKClient(url)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import List

from stytch.b2b.models.discovery import DiscoveredOrganization
from stytch.core.response_base import ResponseBase


class AuthenticateResponse(ResponseBase):
    """Response type for `Discovery.authenticate`.
    Fields:
      - intermediate_session_token: The Intermediate Session Token. This token does not necessarily belong to a specific instance of a Member, but represents a bag of factors that may be converted to a member session. The token can be used with the [OTP SMS Authenticate endpoint](https://stytch.com/docs/b2b/api/authenticate-otp-sms), [TOTP Authenticate endpoint](https://stytch.com/docs/b2b/api/authenticate-totp), or [Recovery Codes Recover endpoint](https://stytch.com/docs/b2b/api/recovery-codes-recover) to complete an MFA flow and log in to the Organization. The token has a default expiry of 10 minutes. It can also be used with the [Exchange Intermediate Session endpoint](https://stytch.com/docs/b2b/api/exchange-intermediate-session) to join a specific Organization that allows the factors represented by the intermediate session token; or the [Create Organization via Discovery endpoint](https://stytch.com/docs/b2b/api/create-organization-via-discovery) to create a new Organization and Member. Intermediate Session Tokens have a default expiry of 10 minutes.
      - email_address: The email address.
      - discovered_organizations: An array of `discovered_organization` objects tied to the `intermediate_session_token`, `session_token`, or `session_jwt`. See the [Discovered Organization Object](https://stytch.com/docs/b2b/api/discovered-organization-object) for complete details.

      Note that Organizations will only appear here under any of the following conditions:
      1. The end user is already a Member of the Organization.
      2. The end user is invited to the Organization.
      3. The end user can join the Organization because:

          a) The Organization allows JIT provisioning.

          b) The Organizations' allowed domains list contains the Member's email domain.

          c) The Organization has at least one other Member with a verified email address with the same domain as the end user (to prevent phishing attacks).
      - provider_type: Denotes the OAuth identity provider that the user has authenticated with, e.g. Google, Microsoft, GitHub etc.
      - provider_tenant_id: The tenant ID returned by the OAuth provider. This is typically used to identify an organization or group within the provider's domain. For example, in HubSpot this is a Hub ID, in Slack this is the Workspace ID, and in GitHub this is an organization ID. This field will only be populated if exactly one tenant ID is returned from a successful OAuth authentication and developers should prefer `provider_tenant_ids` over this since it accounts for the possibility of an OAuth provider yielding multiple tenant IDs.
      - provider_tenant_ids: All tenant IDs returned by the OAuth provider. These is typically used to identify organizations or groups within the provider's domain. For example, in HubSpot this is a Hub ID, in Slack this is the Workspace ID, and in GitHub this is an organization ID. Some OAuth providers do not return tenant IDs, some providers are guaranteed to return one, and some may return multiple. This field will always be populated if at least one tenant ID was returned from the OAuth provider and developers should prefer this field over `provider_tenant_id`.
      - full_name: The full name of the authenticated end user, if available.
    """  # noqa

    intermediate_session_token: str
    email_address: str
    discovered_organizations: List[DiscoveredOrganization]
    provider_type: str
    provider_tenant_id: str
    provider_tenant_ids: List[str]
    full_name: str
